# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
# Load the data
import numpy as np
from scipy.sparse import load_npz
import time
import sys
import argparse

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)

CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)

args = CLI.parse_args()

t0 = time.time()


X_train = load_npz(args.data_path + "/data/criteo.kaggle2014.X_train.npz")
X_test  = load_npz(args.data_path + "/data/criteo.kaggle2014.X_test.npz")
y_train = np.load(args.data_path + "/data/criteo.kaggle2014.y_train.npy")
y_test  = np.load(args.data_path + "/data/criteo.kaggle2014.y_test.npy")
print("Data load time (s): {0:.2f}".format(time.time()-t0))


num_threads = 4
max_iter = 1
cpu_gpu = "CPU"

if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

# Import the LogisticRegression from pai4sk
from pai4sk import LogisticRegression
lr = LogisticRegression(dual=False, max_iter=max_iter, num_threads=num_threads,
                        class_weight=class_weight, privacy=True, eta=0.3, 
                        batch_size=100, privacy_epsilon=10.0, grad_clip=1.0)

# Training
t0 = time.time()
lr.fit(X_train, y_train)
print("[pai4sk] Training time (s):  {1:.2f}".format(cpu_gpu, time.time()-t0))

# Inference
proba_test = lr.predict_proba(X_test)

# Evaluate log-loss on test set
from pai4sk.metrics import log_loss
logloss_snap = log_loss(y_test, proba_test)
print("[pai4sk] Logarithmic loss:   {1:.4f}".format(cpu_gpu, logloss_snap))

# Import the LogisticRegression from pai4sk.linear_model 
from pai4sk.linear_model import LogisticRegression
lr = LogisticRegression(max_iter=max_iter, num_threads=num_threads,
                        class_weight=class_weight, fit_intercept=False, 
                        dual=False,privacy=True, eta=0.3, batch_size=100, 
                        privacy_epsilon=10.0, grad_clip=1.0)
# Training time
t0 = time.time()
lr.fit(X_train, y_train)
print("[pai4sk.lmodel] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
proba_test = lr.predict_proba(X_test)

# Evaluate log-loss on test set
from pai4sk.metrics import log_loss
logloss_pai4sk_lmodel = log_loss(y_test, proba_test)
print("[pai4sk.lmodel] Logarithmic loss:   {0:.4f}".format(logloss_pai4sk_lmodel))

