# Housing Example

In this example we will train Decision Tree and Random Forest models on the housing dataset, using both `scikit-learn` and `snap-ml-local`. 

## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/regression/housing_scale
mv housing_scale housing
cd ../
```

## Data Pre-processing
The data is in SvmLight format. Here we perform some pre-processing. 

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-housing/preprocess-housing.py`.

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-housing/preprocess-housing.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.


## Training and Evaluating a Decision Tree Regressor Model

One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-housing/example-housing-decision-tree.py`

Command to run the example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-housing/example-housing-decision-tree.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

One should see the following output on P9+V100 system:
```
Data load time (s):  0.01
[snap] Training time (s):  1.60
[snap] Mean squared error score:   24.6704
[snap.histogram] Training time (s):  0.45
[snap.histogram] Mean squared error score:   25.4929
[sklearn] Training time (s):  2.74
[sklearn] Mean squared error score:   29.2374
```

For the decision tree and random forest models, `snap-ml-local` provides a feature called `histograms`: when enabled, 
the values of each feature are grouped into a user-defined number of bins; during training, the splits are evaluated at the bin values instead
of the individual feature values. This can significantly accelerate the training of the model. To enable `histograms`, the user needs to define two
parameters at model instantiation time: `use_histograms` and `hist_nbins`. An example of how to use these parameters
can be found in `$CONDA_PREFIX/pai4sk/snap-ml-local/local-examples/example-housing/example-housing-decision-tree.py`. The `sklearn` library 
does not offer (as of version 0.22) a decision tree / random forest API with `histograms` functionality. Nevertheless, in this example,
we compare the `snap-ml-local` `histograms`-based decision tree with the `sklearn` exact decision tree algorithm and show the benefits of 
using `histograms` both in terms of training time and generalization performance.

## Training and Evaluating a Random Forest Regressor Model

One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-housing/example-housing-random-forest.py`

Command to run the example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-housing/example-housing-random-forest.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

One should see the following output on P9+V100 system:
```
Data load time (s):  0.01
[snap] Training time (s): 1.18
[snap] Mean squared error score: 12.3261
[snap.histogram] Training time (s): 0.54
[snap.histogram] Mean squared error score: 12.9413
[sklearn] Training time (s):  1.08
[sklearn] Mean squared error score:   13.2532
```


&copy; Copyright IBM Corporation 2018, 2019
